import { parceable } from 'Schema'
import SchemaItem from '../SchemaItem'

export default class SchemaDate extends SchemaItem<Date> {

	@parceable()
	public parseString(format: 'iso8601' | 'yy-mm-dd' | 'jj/mm/yy' = 'iso8601') {
		switch (format) {
			case 'yy-mm-dd':
			case 'iso8601': {
				this.addPreProcess((it) => {
					if (typeof it !== 'string') {
						return it
					}

					const date = new Date(it)

					if (isNaN(date.getDate())) {
						return it
					}

					return date
				})
				break
			}
			case 'jj/mm/yy': {
				this.addPreProcess((input) => {
					if (typeof input !== 'string') {
						return input
					}
					const splitted = input.split('/').map((it) => Number.parseInt(it, 10))
					if (splitted.length !== 3) {
						return input
					}

					const date = new Date(splitted[2], splitted[1] - 1, splitted[0])
					if (isNaN(date.getDate())) {
						return input
					}

					return date
				})
			}
		}

		return this
	}

	public override isOfType(input: unknown): input is Date {
		return input instanceof Date
	}
}
