import { parceable } from 'Schema'
import SchemaItem from '../SchemaItem'
import { SchemaInfer, ValidationResult } from '../types'

export default class SchemaNullable<Type extends SchemaItem> extends SchemaItem<SchemaInfer<Type> | undefined> {
	public constructor(public readonly child: Type) { super([child]) }

	@parceable()
	public falthyAsNull() {
		this.addPreProcess((it) => !it ? undefined : it)

		return this
	}

	public unwrap(): Type {
		return this.child
	}

	public override parse(input: unknown, options?: { fast?: boolean }): ValidationResult<SchemaInfer<Type> | undefined> {
		const res = super.parse(input, options)
		if (!res.valid) {
			return this.child.parse(input, options)
		}

		if (this.isNull(res.object)) {
			return {
				valid: true,
				object: undefined
			}
		}

		return this.child.parse(input, options)
	}

	public override isOfType(input: unknown): input is SchemaInfer<Type> | undefined {
		return this.isNull(input) || this.child.isOfType(input)
	}

	private isNull(value: unknown): value is undefined | null {
		return typeof value === 'undefined' || value === null
	}
}
