import { isObject, objectLoop } from '@dzeio/object-util'
import SchemaItem from '../SchemaItem'
import { SchemaInfer, ValidationResult } from '../types'

export default class SchemaRecord<Keys extends SchemaItem, Values extends SchemaItem> extends SchemaItem<Record<SchemaInfer<Keys>, SchemaInfer<Values>>> {

	public constructor(private readonly keys: Keys, private readonly values: Values) {
		super([keys, values])
	}

	public parse(input: unknown, options?: { fast?: boolean }): ValidationResult<Record<SchemaInfer<Keys>, SchemaInfer<Values>>> {

		// check errors from itself
		const { valid, object, errors = [] } = super.parse(input, options)

		// skip checking childs if self is not valid (maybe still try to check childs whan fast is false ?)
		if (!valid) {
			return {
				valid,
				object,
				errors
			} as ValidationResult<SchemaInfer<this>>
		}

		const clone: Partial<SchemaInfer<this>> = {}
		objectLoop(object, (value, key: string | number) => {
			const res1 = this.keys.parse(key)
			const res2 = this.values.parse(value)
			if (!res1.valid || !res2.valid) {
				errors.push(...((res1.errors ?? []).concat(...(res2.errors ?? []))).map((it) => ({
					message: it.message,
					field: it.field ? `${key as string}.${it.field}` : key.toString()
				})))
			} else {
				// @ts-expect-error normal behavior
				// eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
				clone[res1.object] = res2.object
			}

			// skip completion if fast is enabled
			return errors.length === 0 || !options?.fast
		})

		// answer !
		return {
			valid: errors.length === 0,
			errors: errors,
			object: clone
		} as ValidationResult<SchemaInfer<this>>
	}

	public override isOfType(input: unknown): input is Record<SchemaInfer<Keys>, SchemaInfer<Values>> {
		return isObject(input) && Object.prototype.toString.call(input) === '[object Object]'
	}
}
